@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2004 Marti Maria
+//  Copyright (C) 1998-2007 Marti Maria
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -23,7 +23,7 @@
 
 #include "lcms.h"
 
-// Gamma handling. I'm encoding gamma as 0..ffff value
+// Gamma handling. 
 
 LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries);
 void         LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma);
@@ -31,11 +31,12 @@ void         LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
 LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma);
 LPGAMMATABLE LCMSEXPORT cmsDupGamma(LPGAMMATABLE Src);
 LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma);
+LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double Params[]);
 LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma, LPGAMMATABLE OutGamma);
 LPGAMMATABLE LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma, LPGAMMATABLE OutGamma, int nPoints);
-BOOL         LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
+LCMSBOOL         LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
 
-BOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nPoints);
+LCMSBOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nPoints);
 
 
 // Sampled curves
@@ -44,7 +45,7 @@ LPSAMPLEDCURVE cdecl cmsAllocSampledCurve(int nItems);
 void           cdecl cmsFreeSampledCurve(LPSAMPLEDCURVE p);
 void           cdecl cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max);
 void           cdecl cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max);
-BOOL           cdecl cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double SmoothingLambda);
+LCMSBOOL       cdecl cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double SmoothingLambda);
 void           cdecl cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoints);
 
 LPSAMPLEDCURVE cdecl cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints);
@@ -54,81 +55,86 @@ double LCMSEXPORT cmsEstimateGammaEx(LPWORD GammaTable, int nEntries, double The
 
 // ----------------------------------------------------------------------------------------
 
-// #define DEBUG 1
 
 #define MAX_KNOTS   4096
 typedef float vec[MAX_KNOTS+1];
 
 
-// default gamma function
-
-#if 1
-
-// No slope limiting
+// Ciclic-redundant-check for assuring table is a true representation of parametric curve
 
+// The usual polynomial, which is used for AAL5, FDDI, and probably Ethernet 
+#define QUOTIENT 0x04c11db7
+    
 static
-double FGamma(double R, double x)
-{
-       return pow(R, x);
+unsigned int Crc32(unsigned int result, LPVOID ptr, int len)
+{    
+    int          i,j;
+    BYTE         octet; 
+    LPBYTE       data = (LPBYTE) ptr;
+        
+    for (i=0; i < len; i++) {
+
+        octet = *data++;
+
+        for (j=0; j < 8; j++) {
+
+            if (result & 0x80000000) {
+
+                result = (result << 1) ^ QUOTIENT ^ (octet >> 7);
+            }
+            else
+            {
+                result = (result << 1) ^ (octet >> 7);
+            }
+            octet <<= 1;
+        }
+    }
+    
+    return result;
 }
 
-#else
+// Get CRC of gamma table 
 
-// Quantization becomes evident below 
-//
-//      R^x * 65535 < 1.0
-//
-// So, we apply linear tram on
-// 
-//      R^x < 1 / 65535
-//
-//      x * log(R) < log(1) - log(65535) 
-//
-//      R < (1 /exp(log(65535) / x)
-//
-//  Somehow arbitrarely, we adopt 0.018, which corresponds
-//  with gamma 2.7 This is adequate for most situations.
-//
-//  The resulting line must be:
-//
-//  f(0) = 0
-//  f(0.018) = pow(0.018, x)
-//
-//  so, 
-//  
-//  f() = pow(0.018, x) / 0.018
-
-static 
-double FGamma(double R, double x)
+unsigned int _cmsCrc32OfGammaTable(LPGAMMATABLE Table)
 {
-       if (R < 0.018) 
-           return (R * pow(0.018, x)) / 0.018;
+    unsigned int crc = ~0U;
+
+    crc = Crc32(crc, &Table -> Seed.Type,  sizeof(int));
+    crc = Crc32(crc, Table ->Seed.Params,  sizeof(double)*10);
+    crc = Crc32(crc, &Table ->nEntries,    sizeof(int));
+    crc = Crc32(crc, Table ->GammaTable,   sizeof(WORD) * Table -> nEntries);
+
+    return ~crc;
 
-       return pow(R, x);
 }
 
 
-#endif
-
 LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries)
 {
        LPGAMMATABLE p;
        size_t size;
 
+       if (nEntries > 65530 || nEntries <= 0) {
+                cmsSignalError(LCMS_ERRC_ABORTED, "Couldn't create gammatable of more than 65530 entries");
+                return NULL;
+       }
+
        size = sizeof(GAMMATABLE) + (sizeof(WORD) * (nEntries-1));
 
-       p = (LPGAMMATABLE) malloc(size);
+       p = (LPGAMMATABLE) _cmsMalloc(size);
        if (!p) return NULL;
 
-       p -> nEntries = nEntries;
-       ZeroMemory(p -> GammaTable, nEntries * sizeof(WORD));
+       ZeroMemory(p, size);
 
+       p -> Seed.Type     = 0;
+       p -> nEntries = nEntries;
+       
        return p;
 }
 
 void LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma)
 {
-       if (Gamma) free(Gamma);
+       if (Gamma)  _cmsFree(Gamma);
 }
 
 
@@ -141,50 +147,6 @@ void LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3])
     Gamma[0] = Gamma[1] = Gamma[2] = NULL;
 }
 
-// Build a gamma table based on gamma constant
-
-LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
-{
-       LPGAMMATABLE p;
-       LPWORD Table;
-       int i;
-       double R, Val;
-
-       if (nEntries > 65530) {
-                cmsSignalError(LCMS_ERRC_WARNING, "Couldn't create gammatable of more than 65530 entries; 65530 assumed");
-                nEntries = 65530;
-       }
-
-       p = cmsAllocGamma(nEntries);
-       if (!p) return NULL;
-
-       Table = p -> GammaTable;
-       if (Gamma == 0.0)
-       {
-              ZeroMemory(Table, nEntries*sizeof(WORD));
-              return p;
-       }
-
-       if (Gamma == 1.0) {
-
-           for (i=0; i < nEntries; i++) {
-
-            Table[i] = _cmsQuantizeVal(i, nEntries);              
-
-           }
-           return p;
-       }
-
-       for (i=0; i < nEntries; i++)
-       {
-              R   = (double) i / (nEntries-1);
-              Val = FGamma(R, Gamma);
-
-              Table[i] = (WORD) floor(Val * 65535. + .5);
-       }
-
-       return p;
-}
 
 
 // Duplicate a gamma table
@@ -192,14 +154,14 @@ LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
 LPGAMMATABLE  LCMSEXPORT cmsDupGamma(LPGAMMATABLE In)
 {
        LPGAMMATABLE Ptr;
+       size_t size;
 
        Ptr = cmsAllocGamma(In -> nEntries);
        if (Ptr == NULL) return NULL;
 
-       CopyMemory(Ptr -> GammaTable,
-                  In -> GammaTable,
-                  In -> nEntries * sizeof(WORD));
+       size = sizeof(GAMMATABLE) + (sizeof(WORD) * (In -> nEntries-1));
 
+       CopyMemory(Ptr, In, size);                  
        return Ptr;
 }
 
@@ -230,7 +192,7 @@ LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma,
 
               wValIn  = cmsLinearInterpLUT16(RGB_8_TO_16(i), InPtr, &L16In);
               wValOut = cmsReverseLinearInterpLUT16(wValIn, OutPtr, &L16Out);
-
+              
               p -> GammaTable[i] = wValOut;
        }
 
@@ -263,6 +225,8 @@ LPGAMMATABLE LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma,
 
     // Does clean "hair"
     cmsSmoothSampledCurve(r, 0.001);
+
+    cmsClampSampledCurve(r, 0.0, 65535.0); 
     
     cmsFreeSampledCurve(x);
     cmsFreeSampledCurve(y);
@@ -284,6 +248,15 @@ LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma
        LPWORD InPtr;
        LPGAMMATABLE p;
 
+       // Try to reverse it analytically whatever possible
+       if (InGamma -> Seed.Type > 0 && InGamma -> Seed.Type <= 5 &&             
+            _cmsCrc32OfGammaTable(InGamma) == InGamma -> Seed.Crc32) {
+
+                return cmsBuildParametricGamma(nResultSamples, -(InGamma -> Seed.Type), InGamma ->Seed.Params);
+       }
+
+
+       // Nope, reverse the table 
        p = cmsAllocGamma(nResultSamples);
        if (!p) return NULL;
 
@@ -299,9 +272,6 @@ LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma
               p -> GammaTable[i] = wValOut;
        }
 
-
-       // Does eliminate "hair" on curve      
-      //  cmsSmoothGamma(p, 0.001);           
            
        return p;
 }
@@ -319,10 +289,15 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
         LPGAMMATABLE Table;
         double R, Val, dval, e;
         int i;
+        int ParamsByType[] = { 0, 1, 3, 4, 5, 7 };
 
         Table = cmsAllocGamma(nEntries);
         if (NULL == Table) return NULL;
 
+        Table -> Seed.Type = Type;       
+
+        CopyMemory(Table ->Seed.Params, Params, ParamsByType[abs(Type)] * sizeof(double));
+
 
         for (i=0; i < nEntries; i++) {
 
@@ -472,9 +447,19 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
         Table->GammaTable[i] = (WORD) floor(dval);
         }
 
+        Table -> Seed.Crc32 = _cmsCrc32OfGammaTable(Table);
+
         return Table;
 }
 
+// Build a gamma table based on gamma constant
+
+LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
+{
+    return cmsBuildParametricGamma(nEntries, 1, &Gamma);
+}
+
+
 
 // From: Eilers, P.H.C. (1994) Smoothing and interpolation with finite
 // differences. in: Graphic Gems IV, Heckbert, P.S. (ed.), Academic press.
@@ -522,7 +507,7 @@ void smooth2(vec w, vec y, vec z, float lambda, int m)
 
 // Smooths a curve sampled at regular intervals
 
-BOOL LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda)
+LCMSBOOL LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda)
 
 {
     vec w, y, z;
@@ -634,13 +619,13 @@ LPSAMPLEDCURVE cmsAllocSampledCurve(int nItems)
 {
     LPSAMPLEDCURVE pOut;
 
-    pOut = (LPSAMPLEDCURVE) malloc(sizeof(SAMPLEDCURVE));
+    pOut = (LPSAMPLEDCURVE) _cmsMalloc(sizeof(SAMPLEDCURVE));
     if (pOut == NULL)
             return NULL;
 
-    if((pOut->Values = (double *) malloc(nItems * sizeof(double))) == NULL)
+    if((pOut->Values = (double *) _cmsMalloc(nItems * sizeof(double))) == NULL)
     {
-        free(pOut);
+         _cmsFree(pOut);
         return NULL;
     }
 
@@ -653,8 +638,8 @@ LPSAMPLEDCURVE cmsAllocSampledCurve(int nItems)
 
 void cmsFreeSampledCurve(LPSAMPLEDCURVE p)
 {
-    free((LPVOID) p -> Values);
-    free((LPVOID) p);
+     _cmsFree((LPVOID) p -> Values);
+     _cmsFree((LPVOID) p);
 }
 
 
@@ -725,7 +710,7 @@ void cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max)
 
 // Smooths a curve sampled at regular intervals
 
-BOOL cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double lambda)
+LCMSBOOL cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double lambda)
 {
     vec w, y, z;
     int i, nItems;
@@ -771,7 +756,8 @@ double ScaleVal(double v, double Min, double Max, int nPoints)
 
         double a, b;
 
-        if (v < 0) return 0;
+        if (v <= Min) return 0;
+        if (v >= Max) return (nPoints-1);
     
         a = (double) (nPoints - 1) / (Max - Min);
         b = a * Min;
@@ -830,7 +816,7 @@ LPSAMPLEDCURVE cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nRes
         // Scale t to x domain
         x = (i * (MaxX - MinX) / (nResultingPoints-1)) + MinX;
 
-        // Find interval in which t is within (always up, 
+        // Find interval in which j is within (always up, 
         // since fn should be monotonic at all)
 
         j = 1;
@@ -904,38 +890,15 @@ LPSAMPLEDCURVE cmsConvertGammaToSampledCurve(LPGAMMATABLE Gamma, int nPoints)
 }
 
 
-#ifdef DEBUG
-static
-void ASAVE(LPWORD Table, int nEntries, const char* dump)
-{
-    FILE* f;
-    int i;
-
-        f = fopen(dump, "wt");
-        if (!f)
-                return;
-
-        
-
-    for (i=0; i < nEntries; i++)
-        fprintf(f, "%g\n", (double) Table[i]);
-        
-
-    fclose(f);
-}
-#endif
 
 
 // Smooth endpoints (used in Black/White compensation)
 
-BOOL _cmsSmoothEndpoints(LPWORD Table, int nEntries)
+LCMSBOOL _cmsSmoothEndpoints(LPWORD Table, int nEntries)
 {
     vec w, y, z;
     int i, Zeros, Poles;
 
-#ifdef DEBUG
-        ASAVE(Table, nEntries, "nonsmt.txt");
-#endif
 
 
     if (cmsIsLinear(Table, nEntries)) return FALSE; // Nothing to do
@@ -987,8 +950,5 @@ BOOL _cmsSmoothEndpoints(LPWORD Table, int nEntries)
         Table[i] = (WORD) floor(v + .5);
         }
 
-#ifdef DEBUG
-        ASAVE(Table, nEntries, "smoothed.txt");
-#endif
     return TRUE;
 }
